# SPDX-FileCopyrightText: 2023 Blender Authors
#
# SPDX-License-Identifier: GPL-2.0-or-later

'''
Usage:
    python discover_nodes.py
        <sources/root>
        <path/to/output.cc>
        <generated_function_name>
        <source>...

The goal is to make it easy for nodes to register themselves without having to have
a central place that registers all nodes manually. A node can use this mechanism by
invoking `NOD_REGISTER_NODE(register_function_name)`.

This scripts finds all those macro invocations generates code that calls the functions.
'''

import os
import re
import sys


def filepath_is_older(filepath_test, filepath_compare):
    import stat
    mtime = os.stat(filepath_test)[stat.ST_MTIME]
    for filepath_other in filepath_compare:
        if mtime < os.stat(filepath_other)[stat.ST_MTIME]:
            return True
    return False


# The build system requires the generated file to be touched if any files used to generate it are newer.
try:
    sys.argv.remove("--use-makefile-workaround")
    use_makefile_workaround = True
except ValueError:
    use_makefile_workaround = False


# NOTE: avoid `pathlib`, pulls in many modules indirectly, path handling is simple enough.
source_root = sys.argv[1]
output_cc_file = sys.argv[2]
function_to_generate = sys.argv[3]
source_cc_files = [
    os.path.join(source_root, path)
    for path in sys.argv[4:]
    if path.endswith(".cc")
]

macro_name = "NOD_REGISTER_NODE"
discover_suffix = "_discover"

include_lines = []
decl_lines = []
func_lines = []

# Add forward declaration to avoid warning.
func_lines.append(f"void {function_to_generate}();")
func_lines.append(f"void {function_to_generate}()")
func_lines.append("{")

# Use a single regular expression to search for opening name-spaces, closing name-spaces
# and macro invocations. This makes it easy to iterate over the matches in order.
re_namespace_begin = r"^namespace ([\w:]+) \{"
re_namespace_end = r"^\}  // namespace ([\w:]+)"
re_macro = r"MACRO\((\w+)\)".replace("MACRO", macro_name)
re_all = f"({re_namespace_begin})|({re_namespace_end})|({re_macro})"
re_all_compiled = re.compile(re_all, flags=re.MULTILINE)

for path in source_cc_files:
    # Read the source code.
    with open(path, "r", encoding="utf-8") as fh:
        code = fh.read()

    # Keeps track of the current name-space we're in.
    namespace_parts = []

    for match in re_all_compiled.finditer(code):
        if entered_namespace := match.group(2):
            # Enter a (nested) name-space.
            namespace_parts += entered_namespace.split("::")
        elif exited_namespace := match.group(4):
            # Exit a (nested) name-space.
            del namespace_parts[-len(exited_namespace.split("::")):]
        elif function_name := match.group(6):
            # Macro invocation in the current name-space.
            namespace_str = "::".join(namespace_parts)
            # Add suffix so that this refers to the function created by the macro.
            auto_run_name = function_name + discover_suffix

            # Declare either outside of any named name-space or in a (nested) name-space.
            # Can't declare it in an anonymous name-space because that would make the
            # declared function static.
            if namespace_str:
                decl_lines.append(f"namespace {namespace_str} {{")
            decl_lines.append(f"void {auto_run_name}();")
            if namespace_str:
                decl_lines.append("}")

            # Call the function.
            func_lines.append(f"  {namespace_str}::{auto_run_name}();")

func_lines.append("}")

# Write the generated code if it changed. If the newly generated code is the same as before,
# don't overwrite the existing file to avoid unnecessary rebuilds.
try:
    with open(output_cc_file, "r", encoding="utf-8") as fh:
        old_generated_code = fh.read()
except:
    old_generated_code = ""
new_generated_code = "\n".join(include_lines + decl_lines + [""] + func_lines)

if old_generated_code != new_generated_code:
    with open(output_cc_file, "w", encoding="utf-8") as fh:
        fh.write(new_generated_code)
elif use_makefile_workaround and filepath_is_older(output_cc_file, (__file__, *source_cc_files)):
    # If the generated file is older than this command, this file would be generated every time.
    os.utime(output_cc_file)



# SPDX-FileCopyrightText: 2016-2020 by Nathan Lovato, Daniel Oakey, Razvan Radulescu, and contributors
#
# SPDX-License-Identifier: GPL-3.0-or-later

import bpy


class PowerSequencerProperties(bpy.types.PropertyGroup):
    playback_speed: bpy.props.EnumProperty(
        items=[
            ("NORMAL", "Normal (1x)", ""),
            ("DOUBLE", "Double (2x)", ""),
            ("TRIPLE", "Triple (3x)", ""),
        ],
        name="Playback",
        default="NORMAL",
    )

    active_tab: bpy.props.StringProperty(
        name="Active Tab", description="The name of the active tab in the UI", default="Sequencer"
    )


def register_properties():
    bpy.utils.register_class(PowerSequencerProperties)
    bpy.types.Scene.power_sequencer = bpy.props.PointerProperty(type=PowerSequencerProperties)


def unregister_properties():
    bpy.utils.unregister_class(PowerSequencerProperties)



# SPDX-FileCopyrightText: 2019-2022 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

bl_info = {
    "name": "Random Vertices",
    "author": "Oscurart, Greg",
    "version": (1, 3),
    "blender": (2, 63, 0),
    "location": "Object > Transform > Random Vertices",
    "description": "Randomize selected components of active object",
    "warning": "",
    "doc_url": "",
    "category": "Mesh",
}


import bpy
from bpy.types import Operator
import random
import bmesh
from bpy.props import (
        BoolProperty,
        FloatProperty,
        IntVectorProperty,
        )


def add_object(self, context, valmin, valmax, factor, vgfilter):
    # select an option with weight map or not
    mode = bpy.context.active_object.mode
    # generate variables
    objact = bpy.context.active_object
    listver = []
    warn_message = False

    # switch to edit mode
    bpy.ops.object.mode_set(mode='OBJECT')
    bpy.ops.object.mode_set(mode='EDIT')

    # bmesh object
    odata = bmesh.from_edit_mesh(objact.data)
    odata.select_flush(False)

    # if the vertex is selected add to the list
    for vertice in odata.verts[:]:
        if vertice.select:
            listver.append(vertice.index)

    # If the minimum value is greater than the maximum,
    # it adds a value to the maximum
    if valmin[0] >= valmax[0]:
        valmax[0] = valmin[0] + 1

    if valmin[1] >= valmax[1]:
        valmax[1] = valmin[1] + 1

    if valmin[2] >= valmax[2]:
        valmax[2] = valmin[2] + 1

    odata.verts.ensure_lookup_table()

    random_factor = factor
    for vertice in listver:
        odata.verts.ensure_lookup_table()
        if odata.verts[vertice].select:
            if vgfilter is True:
                has_group = getattr(objact.data.vertices[vertice], "groups", None)
                vertex_group = has_group[0] if has_group else None
                vertexweight = getattr(vertex_group, "weight", None)
                if vertexweight:
                    random_factor = factor * vertexweight
                else:
                    random_factor = factor
                    warn_message = True

            odata.verts[vertice].co = (
                (((random.randrange(valmin[0], valmax[0], 1)) * random_factor) / 1000) +
                odata.verts[vertice].co[0],
                (((random.randrange(valmin[1], valmax[1], 1)) * random_factor) / 1000) +
                odata.verts[vertice].co[1],
                (((random.randrange(valmin[2], valmax[2], 1)) * random_factor) / 1000) +
                odata.verts[vertice].co[2]
                )

    if warn_message:
        self.report({'WARNING'},
                    "Some of the Selected Vertices don't have a Group with Vertex Weight assigned")
    bpy.ops.object.mode_set(mode=mode)


class MESH_OT_random_vertices(Operator):
    bl_idname = "mesh.random_vertices"
    bl_label = "Random Vertices"
    bl_description = ("Randomize the location of vertices by a specified\n"
                      "Multiplier Factor and random values in the defined range\n"
                      "or a multiplication of them and the Vertex Weights")
    bl_options = {'REGISTER', 'UNDO'}

    vgfilter: BoolProperty(
            name="Vertex Group",
            description="Use Vertex Weight defined in the Active Group",
            default=False
            )
    factor: FloatProperty(
            name="Factor",
            description="Base Multiplier of the randomization effect",
            default=1
            )
    valmin: IntVectorProperty(
            name="Min XYZ",
            description="Define the minimum range of randomization values",
            default=(0, 0, 0)
            )
    valmax: IntVectorProperty(
            name="Max XYZ",
            description="Define the maximum range of randomization values",
            default=(1, 1, 1)
            )

    @classmethod
    def poll(cls, context):
        return (context.object and context.object.type == "MESH" and
               context.mode == "EDIT_MESH")

    def execute(self, context):
        add_object(self, context, self.valmin, self.valmax, self.factor, self.vgfilter)

        return {'FINISHED'}


# Registration

def register():
    bpy.utils.register_class(MESH_OT_random_vertices)


def unregister():
    bpy.utils.unregister_class(MESH_OT_random_vertices)


if __name__ == '__main__':
    register()



#Starting from 1876 (first type of commercial lamps developed with electricity)
#Carbon arc lamps were being phased out after the 1910s.
#For general lighting the lamp was replaced by the 1920s and 30s in most cities.
#The lamp continued to be used for spot lights, film production lighting and film projector lamps.
#Most of the remaining carbon arc lamps ceased production by the 1980s

import bpy
bpy.context.object.data.type = 'SPOT'
lampdata = bpy.context.object.data

#lampdata.use_halo = True
lampdata.show_cone = True
lampdata.spot_size = 1.5
lampdata.spot_blend = 0.3
lampdata.color = (1.0, 0.9803921580314636, 0.95686274766922)
lampdata.energy = 51.29162#55000lm/21.446(=lux)*0.004*2.5(distance) *2 for distance is the point of half strength



import bpy
material = bpy.context.material #(bpy.context.material.active_node_material if bpy.context.material.active_node_material else bpy.context.material)

material.pov_subsurface_scattering.radius = 9.436, 3.348, 1.790
material.pov_subsurface_scattering.color = 0.439, 0.216, 0.141
